import {Entity, PrimaryColumn, Column, ManyToOne, JoinColumn} from "typeorm";
import * as Joi from 'joi';
import {Prisoner} from "../Entities/Prisoner";
// create table JUDICIAL_DECISION(
//     DECISION_TYPE_NUMBER varchar(1),
//     PRISON_FILE_NUMBER varchar(10),
//     DATE_OF_DECISION Date,
//     constraint JUDICIAL_DECISION_key primary key(DECISION_TYPE_NUMBER,PRISON_FILE_NUMBER,DATE_OF_DECISION),
//     constraint JUDICIAL_DECISION_fk foreign key(PRISON_FILE_NUMBER) references PRISONER(PRISON_FILE_NUMBER));
@Entity()
export class JudicialDecision {
    @PrimaryColumn({length: 1})
    decision_type_number: string;
    @PrimaryColumn({length: 10})
    prison_file_number: string;
    @PrimaryColumn()
    date_of_decision: Date;
    // '{primary: false}' => non unique...
    @ManyToOne(type => Prisoner, prisoner => prisoner.judicial_decision, {nullable: false})
    @JoinColumn({name: "PRISON_FILE_NUMBER"})
    prisoner: Prisoner;
}

export const judicial_decision_Joi_schema = Joi.object().keys({
    decision_type_number: Joi.string().required().valid('1', '2', '3').max(1),
    prison_file_number: Joi.string().required().max(10),
    date_of_decision: Joi.date().required(),
    date_of_final_discharge: Joi.date().optional(),
    duration: Joi.number().optional().integer().greater(0)
}).xor('date_of_final_discharge', 'duration');

@Entity()
export class Conviction extends JudicialDecision {
    @Column()
    duration: number;
}

@Entity()
export class ShortenedSentence extends JudicialDecision {
    @Column()
    duration: number;
}

@Entity()
export class FinalDischarge extends JudicialDecision {
    @Column()
    date_of_final_discharge: Date;
}
