import * as http from "http";

import * as Express from 'express';
import {ConnectionOptions, DataSource} from "typeorm";

import {CriminalCase_service, All_criminal_cases} from './Services/CriminalCaseCRUD';
import {JudicialDecision_service, All_judicial_decisions, All_convictions} from './Services/JudicialDecisionCRUD';
import {Motive_service, All_motives} from './Services/MotiveCRUD';
import {Prisoner_service, All_prisoners} from './Services/PrisonerCRUD';
import {Use_cases} from './Services/Use_cases';

// https://github.com/typeorm/typeorm/blob/master/docs/connection-options.md#what-is-connectionoptions
const options: ConnectionOptions = {
    type: "sqlite",
    database: "./New_York_City_Penitentiary_database",
    entities: [
        __dirname + "/Entities/*.js"
    ],
    logging: false,
    synchronize: false // Don't overwrite existing schema (and data)...
};

const New_York_City_Penitentiary_database = new DataSource(options);
New_York_City_Penitentiary_database.connect().then(connection => {
    const web = Express();
    web.use(Express.json());
    web.listen(1963);
    // Launch services:
    CriminalCase_service(web, connection);
    JudicialDecision_service(web, connection);
    Motive_service(web, connection);
    Prisoner_service(web, connection);
    Use_cases(web, connection);
    /** TEST */
    const B25000Besancon = JSON.stringify({
        criminal_case_number: "B25000",
        jurisdiction_name: escape("Besancon"),
        date_of_criminal_case: "1964-06-28"
    });
    let request = http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/CriminalCase',
        method: 'POST',
        headers: {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': B25000Besancon.length
        }
    }, response => {
        console.log(`\tstatusCode: ${response.statusCode}`)
        response.on('data', result => {
            process.stdout.write(result + '\n')
        })
    });
    request.on('error', error => {
        console.error("Error: " + error.message + '\n');
    });
    request.write(B25000Besancon);
    request.end();

    const Incarceration = JSON.stringify([{
        prison_file_number: "1993",
        given_name: "Barbier--Darnal",
        surname: "Oscar",
        date_of_birth: "1993-11-6",
        place_of_birth: "Nantes",
        date_of_incarceration: "2021-04-22"
    }, {
        criminal_case_number: "P64000",
        jurisdiction_name: "Pau",
        date_of_criminal_case: "2001-01-22"
    }, {
        motive_number: "15",
        motive_label: "procuring for prostitutes"
    }]);
    const Incarcerate = {
        hostname: 'localhost',
        port: 1963,
        path: '/Incarcerate',
        method: 'POST',
        headers: {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': Incarceration.length
        }
    }
    request = http.request(Incarcerate, response => {
        console.log(`\tstatusCode: ${response.statusCode}`)
        response.on('data', result => {
            process.stdout.write(result + '\n');
        })
    });
    request.on('error', error => {
        console.error("Error: " + error.message + '\n');
    });
    request.write(Incarceration);
    request.end();

    // All_criminal_cases();
    // All_motives();
    // All_prisoners();

    http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/Under_remand',
        method: 'GET'
    }, (response) => {
        let data = '';
        response.on('data', chunk => { // A chunk of data has been received...
            data += chunk;
        });
        response.on('end', () => { // The whole response has been received...
            let prisoners;
            try {
                prisoners = JSON.parse(data);
                console.log(JSON.stringify(prisoners));
            } catch (exception) {
                console.log(`${exception.name}: ${exception.message}`);
            }
        });
    }).end();

    const conviction = JSON.stringify({ // Hollande is no longer under remand...
        prison_file_number: "2012",
        date_of_decision: "2021-04-24",
        duration: 100
    });
    request = http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/Take_conviction_decision',
        method: 'POST',
        headers: {
            'Content-Type': 'application/json; charset=utf-8',
            'Content-Length': conviction.length
        }
    }, response => {
        console.log(`\tstatusCode: ${response.statusCode}`)
        response.on('data', result => {
            process.stdout.write(result + '\n')
        })
    });
    request.on('error', error => {
        console.error("Error: " + error.message + '\n');
    });
    request.write(conviction);
    request.end();

    // All_judicial_decisions();
    // All_convictions();
    /** End of TEST */
});









