import * as http from "http";

import {CriminalCase, criminal_case_Joi_schema} from '../Entities/CriminalCase';

export const CriminalCase_service = (web, database) => {
    // GET: localhost:1963/CriminalCase 
    web.get('/CriminalCase', async (request, response) => {
        const criminal_cases = await database.getRepository(CriminalCase).find();
        response.send(criminal_cases);
    });
    // GET: localhost:1963/CriminalCase/{criminal_case_number}/{jurisdiction_name}
    web.get('/CriminalCase/:criminal_case_number', async (request, response) => {
        const criminal_case = await database.getRepository(CriminalCase).findOne({
            criminal_case_number: request.params.criminal_case_number,
            jurisdiction_name: request.params.jurisdiction_name
        });
        response.send(criminal_case);
    });
    // POST: localhost:1963/CriminalCase 
    web.post('/CriminalCase', async (request, response) => {
        const criminal_case: CriminalCase = request.body;
        const {error, value} = criminal_case_Joi_schema.validate({
            criminal_case_number: criminal_case.criminal_case_number,
            jurisdiction_name: criminal_case.jurisdiction_name,
            date_of_criminal_case: criminal_case.date_of_criminal_case
        });
        if (error === undefined) {
            await database.getRepository(CriminalCase).save(criminal_case);
            await response.status(201).json({
                message: "CriminalCase inserted with success..."
            });
        } else {
            await response.status(400).json({
                message: "CriminalCase not inserted: " + error.message
            });
        }
    });
    // DELETE: localhost:1963/CriminalCase/{criminal_case_number}/{jurisdiction_name}
    web.delete('/CriminalCase/:criminal_case_number/:jurisdiction_name', async (request, response) => {
        await database.getRepository(CriminalCase).delete({
            criminal_case_number: request.params.criminal_case_number,
            jurisdiction_name: request.params.jurisdiction_name
        });
        response.status(201).json({
            message: "CriminalCase removed with success..."
        });
    });
    // PUT: localhost:1963/CriminalCase
    web.patch('/CriminalCase', async (request, response) => {
        const criminal_case: CriminalCase = request.body;
        const {error, value} = criminal_case_Joi_schema.validate({
            criminal_case_number: criminal_case.criminal_case_number,
            jurisdiction_name: criminal_case.jurisdiction_name,
            date_of_criminal_case: criminal_case.date_of_criminal_case
        });
        if (error === undefined) {
            await database.getRepository(CriminalCase).save(criminal_case);
            await response.status(201).json({
                message: "CriminalCase updated with success..."
            });
        } else {
            await response.status(400).json({
                message: "CriminalCase not updated: " + error.message
            });
        }
    });
};

export const All_criminal_cases = () => {
    http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/CriminalCase',
        method: 'GET'
    }, (response) => {
        let data = '';
        response.on('data', chunk => { // A chunk of data has been received...
            data += chunk;
        });
        response.on('end', () => { // The whole response has been received...
            let criminal_cases;
            try {
                criminal_cases = JSON.parse(data);
                console.log(criminal_cases);
            } catch (exception) {
                console.log(`${exception.name}: ${exception.message}`);
            }
        });
    }).end();
}
