import * as http from "http";

import {Motive, motive_Joi_schema} from '../Entities/Motive';

export const Motive_service = (web, database) => {
    // GET: localhost:80/Motive
    web.get('/Motive', async (request, response) => {
        const motives = await database.getRepository(Motive).find();
        response.send(motives);
    });
    // GET: localhost:80/Motive/{motive_number}
    web.get('/Motive/:motive_number', async (request, response) => {
        const motive = await database.getRepository(Motive).findOne(request.params.motive_number);
        response.send(motive);
    });
    // POST: localhost:80/Motive
    web.post('/Motive', async (request, response) => {
        const motive: Motive = request.body;
        const {error, value} = motive_Joi_schema.validate({
            motive_number: motive.motive_number,
            motive_label: motive.motive_label,
        });
        if (error === undefined) {
            await database.getRepository(Motive).save(motive);
            await response.status(201).json({
                message: "Motive inserted with success..."
            });
        } else {
            await response.status(400).json({
                message: "Motive not inserted: " + error.message
            });
        }
    });
    // DELETE: localhost:80/Motive/{motive_number}
    web.delete('/Motive/:motive_number', async (request, response) => {
        await database.getRepository(Motive).delete(request.params.motive_number);
        await response.status(201).json({
            message: "Motive removed with success..."
        });
    });
    // PUT: localhost:80/Motive
    web.patch('/Motive', async (request, response) => {
        const motive: Motive = request.body;
        const {error, value} = motive_Joi_schema.validate({
            motive_number: motive.motive_number,
            motive_label: motive.motive_label
        });
        if (error === undefined) {
            await database.getRepository(Motive).save(motive);
            await response.status(201).json({
                message: "Motive updated with success..."
            });
        } else {
            await response.status(400).json({
                message: "Motive not updated: " + error.message
            });
        }
    });
}

export const All_motives = () => {
    http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/Motive',
        method: 'GET'
    }, (response) => {
        let data = '';
        response.on('data', chunk => { // A chunk of data has been received...
            data += chunk;
        });
        response.on('end', () => { // The whole response has been received...
            let motives;
            try {
                motives = JSON.parse(data);
                console.log(motives);
            } catch (exception) {
                console.log(`${exception.name}: ${exception.message}`);
            }
        });
    }).end();
}
