import * as http from "http";

import {Prisoner, prisoner_Joi_schema} from '../Entities/Prisoner';

export const Prisoner_service = (web, database) => {
    // GET: localhost:80/Prisoner
    web.get('/Prisoner', async (request, response) => {
        const prisoners = await database.getRepository(Prisoner).find({relations: ["incarceration_main", "incarceration_motive", "judicial_decision"]});
        response.send(prisoners);
    });
    // GET: localhost:80/Prisoner/{prison_file_number} 
    web.get('/Prisoner/:prison_file_number', async (request, response) => {
        const prisoner = await database.getRepository(Prisoner).findOne(request.params.prison_file_number, {relations: ["incarceration_main", "incarceration_motive", "judicial_decision"]});
        response.send(prisoner);
    });
    // DELETE: localhost:80/Prisoner/{prison_file_number} 
    web.delete('/Prisoner/:prison_file_number', async (request, response) => {
        await database.getRepository(Prisoner).delete(request.params.prison_file_number);
        await response.status(201).json({
            message: "Prisoner removed with success..."
        });
    });
    // PUT: localhost:80/Prisoner 
    web.patch('/Prisoner', async (request, response) => {
        const prisoner: Prisoner = request.body;
        const {error, value} = prisoner_Joi_schema.validate({
            prison_file_number: prisoner.prison_file_number,
            given_name: prisoner.given_name,
            surname: prisoner.surname,
            date_of_birth: prisoner.date_of_birth,
            place_of_birth: prisoner.place_of_birth,
            date_of_incarceration:prisoner.date_of_incarceration
        });
        if (error === undefined) {
            await database.getRepository(Prisoner).save(prisoner);
            await response.status(201).json({
                message: "Prisoner updated with success..."
            });
        } else {
            await response.status(400).json({
                message: "Prisoner not updated: " + error.message
            });
        }
    });
}

export const All_prisoners = () => {
    http.request({
        hostname: '127.0.0.1',
        port: 1963,
        path: '/Prisoner',
        method: 'GET'
    }, (response) => {
        let data = '';
        response.on('data', chunk => { // A chunk of data has been received...
            data += chunk;
        });
        response.on('end', () => { // The whole response has been received...
            let prisoners;
            try {
                prisoners = JSON.parse(data);
                console.log(prisoners);
            } catch (exception) {
                console.log(`${exception.name}: ${exception.message}`);
            }
        });
    }).end();
}
