import {CriminalCase, criminal_case_Joi_schema} from '../Entities/CriminalCase';
import {JudicialDecision, judicial_decision_Joi_schema, Conviction} from '../Entities/JudicialDecision';
import {Motive, motive_Joi_schema} from '../Entities/Motive';
import {Prisoner, prisoner_Joi_schema} from '../Entities/Prisoner';

export const Use_cases = (web, database) => {
    web.post('/Incarcerate', async (request, response) => {
        const motive: Motive = request.body[2];
        const criminal_case: CriminalCase = request.body[1];
        const prisoner: Prisoner = request.body[0];
        prisoner.incarceration_main = criminal_case;
        prisoner.incarceration_motive = motive;
        const motive_ = await database.getRepository(Motive).findOne({where: {motive_number: motive.motive_number}});
        if (!motive_) {
            const {error, value} = motive_Joi_schema.validate({
                motive_number: motive.motive_number,
                motive_label: motive.motive_label,
            });
            if (!error)
                await database.getRepository(Motive).save(motive);
        }
        const criminal_case_ = await database.getRepository(CriminalCase).findOne({
            where: {
                criminal_case_number: criminal_case.criminal_case_number,
                jurisdiction_name: criminal_case.jurisdiction_name
            }
        });
        if (!criminal_case_) {
            const {error, value} = criminal_case_Joi_schema.validate({
                criminal_case_number: criminal_case.criminal_case_number,
                jurisdiction_name: criminal_case.jurisdiction_name,
                date_of_criminal_case: criminal_case.date_of_criminal_case
            });
            if (!error)
                await database.getRepository(CriminalCase).save(criminal_case);
        }
        const {error, value} = prisoner_Joi_schema.validate({
            prison_file_number: prisoner.prison_file_number,
            given_name: prisoner.given_name,
            surname: prisoner.surname,
            date_of_birth: prisoner.date_of_birth,
            place_of_birth: prisoner.place_of_birth,
            date_of_incarceration: prisoner.date_of_incarceration
        });
        if (!error) {
            await database.getRepository(Prisoner).save(prisoner);
            await response.status(201).json({
                message: "Incarcerate " + prisoner.prison_file_number + " succeeds..."
            });
        } else {
            await response.status(400).json({
                message: "Incarcerate " + prisoner.prison_file_number + " failed... " + error.message
            });
        }
    });
    // https://github.com/typeorm/typeorm/blob/master/docs/select-query-builder.md#using-subqueries
    web.get('/Under_remand', async (request, response) => { // 'web' is an Express app.
        // Ver. 1:
        // const prisoners: Prisoner[] = await database.getRepository(Prisoner).find({relations: ["judicial_decision"]});
        // response.send(prisoners.filter(prisoner => prisoner.judicial_decision.filter(judicial_decision => judicial_decision.decision_type_number === "1").length === 0));
        // Ver. 2 ('SELECT * FROM Prisoner WHERE prison_file_number NOT IN (SELECT prison_file_number FROM Conviction)'):
        const sub_query = await database.getRepository(Conviction).createQueryBuilder('conviction').select("conviction.prison_file_number");
        const result = await database.getRepository(Prisoner).createQueryBuilder('prisoner')
            .where("prisoner.prison_file_number NOT IN (" + sub_query.getQuery() + ")")
            .getMany();
        response.send(result);
    });

    web.post('/Take_conviction_decision', async (request, response) => {
        const conviction: Conviction = request.body;
        conviction.decision_type_number = '1';
        const {error, value} = judicial_decision_Joi_schema.validate({
            decision_type_number: conviction.decision_type_number,
            prison_file_number: conviction.prison_file_number,
            date_of_decision: conviction.date_of_decision,
            duration: conviction.duration
        });
        if (!error) {
            await database.getRepository(JudicialDecision).save(conviction);
            await database.getRepository(Conviction).save(conviction);
            await response.status(201).json({
                message: "Conviction for " + conviction.prison_file_number + " inserted with success..."
            });
        } else {
            await response.status(400).json({
                message: "Conviction for " + conviction.prison_file_number + " not inserted: " + error.message
            });
        }
    });
}
